@@ -131,6 +131,62 @@
 //!
 //! assert_eq!(checksum, 0xcbf43926);
 //! ```
+//!
+//! # no_std Support
+//!
+//! Supports `no_std` environments. Use `default-features = false` in Cargo.toml.
+//!
+//! Note: When using this library in a `no_std` environment, the final binary must provide:
+//! - A `#[panic_handler]` (e.g., via the `panic-halt` crate)
+//! - A `#[global_allocator]` if using the `alloc` feature
+
+// Provide a panic handler for no_std library checks
+// Disabled with default-features = false (which binaries should use)
+#[cfg(all(
+    feature = "panic-handler",
+    not(feature = "std"),
+    not(test),
+    not(doctest)
+))]
+#[panic_handler]
+fn panic(_info: &core::panic::PanicInfo) -> ! {
+    loop {}
+}
+
+// Provide a global allocator for no_std + alloc library checks
+// Disabled with default-features = false (which binaries should use)
+#[cfg(all(
+    feature = "panic-handler",
+    feature = "alloc",
+    not(feature = "std"),
+    not(test),
+    not(doctest)
+))]
+#[global_allocator]
+static ALLOCATOR: StubAllocator = StubAllocator;
+
+#[cfg(all(
+    feature = "panic-handler",
+    feature = "alloc",
+    not(feature = "std"),
+    not(test),
+    not(doctest)
+))]
+struct StubAllocator;
+
+#[cfg(all(
+    feature = "panic-handler",
+    feature = "alloc",
+    not(feature = "std"),
+    not(test),
+    not(doctest)
+))]
+unsafe impl core::alloc::GlobalAlloc for StubAllocator {
+    unsafe fn alloc(&self, _layout: core::alloc::Layout) -> *mut u8 {
+        core::ptr::null_mut()
+    }
+    unsafe fn dealloc(&self, _ptr: *mut u8, _layout: core::alloc::Layout) {}
+}
 
 use crate::crc32::consts::{
     CRC32_AIXM, CRC32_AUTOSAR, CRC32_BASE91_D, CRC32_BZIP2, CRC32_CD_ROM_EDC, CRC32_CKSUM,
@@ -138,6 +194,7 @@ use crate::crc32::consts::{
 };
 
 #[cfg(any(target_arch = "x86", target_arch = "x86_64", target_arch = "aarch64"))]
+#[cfg(feature = "std")]
 use crate::crc32::fusion;
 
 use crate::crc64::consts::{
@@ -145,13 +202,25 @@ use crate::crc64::consts::{
 };
 use crate::structs::Calculator;
 use crate::traits::CrcCalculator;
-use digest::{DynDigest, InvalidBufferSize};
+#[cfg(feature = "alloc")]
+use digest::DynDigest;
+#[cfg(feature = "alloc")]
+use digest::InvalidBufferSize;
 
+#[cfg(any(target_arch = "x86", target_arch = "x86_64", target_arch = "aarch64"))]
+use crate::feature_detection::get_arch_ops;
 #[cfg(feature = "std")]
 use std::fs::File;
 #[cfg(feature = "std")]
 use std::io::{Read, Write};
 
+#[cfg(all(feature = "alloc", not(feature = "std")))]
+extern crate alloc;
+#[cfg(all(feature = "alloc", not(feature = "std")))]
+use alloc::boxed::Box;
+#[cfg(all(feature = "alloc", not(feature = "std")))]
+use alloc::string::String;
+
 mod algorithm;
 mod arch;
 mod cache;
@@ -161,6 +230,7 @@ mod crc32;
 mod crc64;
 mod enums;
 mod feature_detection;
+#[cfg(feature = "ffi")]
 mod ffi;
 mod generate;
 mod structs;
@@ -326,6 +396,7 @@ pub struct Digest {
     calculator: CalculatorFn,
 }
 
+#[cfg(feature = "alloc")]
 impl DynDigest for Digest {
     #[inline(always)]
     fn update(&mut self, data: &[u8]) {
@@ -819,13 +890,26 @@ pub fn checksum_combine_with_params(
 /// // "x86_64-avx512-vpclmulqdq" - x86_64 with VPCLMULQDQ support
 /// // "x86_64-sse-pclmulqdq" - x86_64 baseline with SSE4.1 and PCLMULQDQ
 /// ```
+#[cfg(all(
+    feature = "alloc",
+    any(target_arch = "aarch64", target_arch = "x86", target_arch = "x86_64")
+))]
 pub fn get_calculator_target(_algorithm: CrcAlgorithm) -> String {
-    use crate::feature_detection::get_arch_ops;
-
     let arch_ops = get_arch_ops();
     arch_ops.get_target_string()
 }
 
+/// Fallback version of get_calculator_target for unsupported architectures
+#[cfg(all(
+    feature = "alloc",
+    not(any(target_arch = "aarch64", target_arch = "x86", target_arch = "x86_64"))
+))]
+pub fn get_calculator_target(_algorithm: CrcAlgorithm) -> String {
+    extern crate alloc;
+    use alloc::string::ToString;
+    "software-fallback-tables".to_string()
+}
+
 /// Returns the calculator function and parameters for the specified CRC algorithm.
 #[inline(always)]
 fn get_calculator_params(algorithm: CrcAlgorithm) -> (CalculatorFn, CrcParams) {
@@ -864,16 +948,35 @@ fn get_calculator_params(algorithm: CrcAlgorithm) -> (CalculatorFn, CrcParams) {
 /// fusion techniques to accelerate the calculation beyond what SIMD can do alone.
 #[inline(always)]
 fn crc32_iscsi_calculator(state: u64, data: &[u8], _params: CrcParams) -> u64 {
-    // both aarch64 and x86 have native CRC-32/ISCSI support, so we can use fusion
-    #[cfg(any(target_arch = "aarch64", target_arch = "x86_64", target_arch = "x86"))]
-    return fusion::crc32_iscsi(state as u32, data) as u64;
+    #[cfg(all(target_arch = "aarch64", feature = "std"))]
+    {
+        use crate::feature_detection::PerformanceTier;
+
+        let arch_ops = get_arch_ops();
+        match arch_ops.get_tier() {
+            PerformanceTier::AArch64AesSha3 | PerformanceTier::AArch64Aes => {
+                return fusion::crc32_iscsi(state as u32, data) as u64;
+            }
+            _ => {}
+        }
+    }
+
+    #[cfg(all(any(target_arch = "x86_64", target_arch = "x86"), feature = "std"))]
+    {
+        use crate::feature_detection::PerformanceTier;
+
+        let arch_ops = get_arch_ops();
+        match arch_ops.get_tier() {
+            PerformanceTier::X86_64Avx512Vpclmulqdq
+            | PerformanceTier::X86_64Avx512Pclmulqdq
+            | PerformanceTier::X86_64SsePclmulqdq
+            | PerformanceTier::X86SsePclmulqdq => {
+                return fusion::crc32_iscsi(state as u32, data) as u64;
+            }
+            _ => {}
+        }
+    }
 
-    #[cfg(all(
-        not(target_arch = "aarch64"),
-        not(target_arch = "x86_64"),
-        not(target_arch = "x86")
-    ))]
-    // Fallback to traditional calculation for other architectures
     Calculator::calculate(state, data, _params)
 }
 
@@ -884,13 +987,19 @@ fn crc32_iscsi_calculator(state: u64, data: &[u8], _params: CrcParams) -> u64 {
 /// so we use the traditional calculation.
 #[inline(always)]
 fn crc32_iso_hdlc_calculator(state: u64, data: &[u8], _params: CrcParams) -> u64 {
-    // aarch64 CPUs have native CRC-32/ISO-HDLC support, so we can use the fusion implementation
-    #[cfg(target_arch = "aarch64")]
-    return fusion::crc32_iso_hdlc(state as u32, data) as u64;
+    #[cfg(all(target_arch = "aarch64", feature = "std"))]
+    {
+        use crate::feature_detection::{get_arch_ops, PerformanceTier};
+        let arch_ops = get_arch_ops();
+
+        match arch_ops.get_tier() {
+            PerformanceTier::AArch64AesSha3 | PerformanceTier::AArch64Aes => {
+                return fusion::crc32_iso_hdlc(state as u32, data) as u64;
+            }
+            _ => {}
+        }
+    }
 
-    // x86 CPUs don't have native CRC-32/ISO-HDLC support, so there's no fusion to be had, use
-    // traditional calculation
-    #[cfg(not(target_arch = "aarch64"))]
     Calculator::calculate(state, data, _params)
 }
 
@@ -1082,6 +1191,16 @@ mod lib {
     }
 
     #[test]
+    fn test_1024_length() {
+        for config in TEST_ALL_CONFIGS {
+            test_length(1024, config);
+        }
+    }
+
+    /// Skipping for Miri runs due to time constraints, underlying code already covered by other
+    /// tests.
+    #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_small_all_lengths() {
         for config in TEST_ALL_CONFIGS {
             // Test each length from 1 to 255
@@ -1091,7 +1210,10 @@ mod lib {
         }
     }
 
+    /// Skipping for Miri runs due to time constraints, underlying code already covered by other
+    /// tests.
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_medium_lengths() {
         for config in TEST_ALL_CONFIGS {
             // Test each length from 256 to 1024, which should fold and include handling remainders
@@ -1101,7 +1223,10 @@ mod lib {
         }
     }
 
+    /// Skipping for Miri runs due to time constraints, underlying code already covered by other
+    /// tests.
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_large_lengths() {
         for config in TEST_ALL_CONFIGS {
             // Test 1 MiB just before, at, and just after the folding boundaries
@@ -1198,7 +1323,10 @@ mod lib {
         );
     }
 
+    /// Skipping for Miri runs due to isolation constraints, underlying code other than I/O already
+    /// covered by other tests.
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_checksum_file() {
         // Create a test file with repeating zeros
         let test_file_path = "test/test_crc32_hash_file.bin";
@@ -1216,7 +1344,10 @@ mod lib {
         std::fs::remove_file(test_file_path).unwrap();
     }
 
+    /// Skipping for Miri runs due to isolation constraints, underlying code other than I/O already
+    /// covered by other tests.
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_checksum_file_with_custom_params() {
         crate::cache::clear_cache();
 
@@ -1264,7 +1395,10 @@ mod lib {
         assert_eq!(result, check);
     }
 
+    /// Skipping for Miri runs due to isolation constraints, underlying code other than I/O already
+    /// covered by other tests.
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_writer() {
         // Create a test file with repeating zeros
         let test_file_path = "test/test_crc32_writer_file.bin";
@@ -1359,6 +1493,7 @@ mod lib {
 
     /// Tests whether the FFI header is up-to-date
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_ffi_header() -> Result<(), String> {
         #[cfg(target_os = "windows")]
         {
@@ -1467,6 +1602,7 @@ mod lib {
     }
 
     #[test]
+    #[allow(clippy::needless_range_loop)] // Intentionally testing indexed get_key() method
     fn test_crc_keys_storage_fold_256() {
         let test_keys = [
             1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,
@@ -1488,6 +1624,7 @@ mod lib {
     }
 
     #[test]
+    #[allow(clippy::needless_range_loop)] // Intentionally testing indexed get_key() method
     fn test_crc_keys_storage_future_test() {
         let test_keys = [
             1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24,
@@ -1510,6 +1647,7 @@ mod lib {
     }
 
     #[test]
+    #[allow(clippy::needless_range_loop)] // Intentionally testing indexed get_key() and get_key_checked() methods
     fn test_crc_params_safe_accessors() {
         // Create a test CrcParams with known keys
         let test_keys = [
