@@ -501,7 +501,7 @@ macro_rules! __ctor_link_section {
             target_vendor = "apple",
             target_family = "wasm",
             target_arch = "xtensa",
-            windows
+            target_vendor = "pc"
         )))]
         compile_error!("#[ctor]/#[dtor] is not supported on the current target");
     }
@@ -533,7 +533,11 @@ macro_rules! __ctor_link_section_attr {
                 ), ".init_array"],
                 [target_arch = "xtensa", ".ctors"],
                 [target_vendor = "apple", "__DATA,__mod_init_func,mod_init_funcs"],
-                [windows, ".CRT$XCU"]],
+                [all(target_vendor = "pc", any(target_env = "gnu", target_env = "msvc")), ".CRT$XCU"],
+                // cygwin support: rustc 1.85 does not like the explicit target_os = "cygwin" condition (https://github.com/mmastrac/rust-ctor/issues/356)
+                // We can work around this by excluding gnu and msvc target envs
+                [all(target_vendor = "pc", not(any(target_env = "gnu", target_env = "msvc"))), ".ctors"]
+                ],
                 #[$used]
                 $item
             );
