@@ -126,18 +126,23 @@ fn test_currency() {
     options.strength = Some(Strength::Quaternary);
 
     let collator = Collator::try_new(Default::default(), options).unwrap();
-    // Iterating as chars and re-encoding due to
-    // https://github.com/rust-lang/rust/issues/83871 being nightly-only. :-(
-    let mut lower_buf = [0u8; 4];
-    let mut higher_buf = [0u8; 4];
-    let mut chars = currencies.chars();
-    while let Some(lower) = chars.next() {
-        let tail = chars.clone();
-        for higher in tail {
-            let lower_str = lower.encode_utf8(&mut lower_buf);
-            let higher_str = higher.encode_utf8(&mut higher_buf);
-            assert_eq!(collator.compare(lower_str, higher_str), Ordering::Less);
+    let mut chars = currencies.char_indices();
+    let mut lower_start = chars.offset();
+    while let Some(_) = chars.next() {
+        let lower_end = chars.offset();
+        let higher_start = lower_end;
+        let mut tail = chars.clone();
+        while let Some(_) = tail.next() {
+            let higher_end = tail.offset();
+            assert_eq!(
+                collator.compare(
+                    &currencies[lower_start..lower_end],
+                    &currencies[higher_start..higher_end]
+                ),
+                Ordering::Less
+            );
         }
+        lower_start = higher_start;
     }
 }
 
@@ -2014,6 +2019,73 @@ fn test_middle_contraction_markers() {
     );
 }
 
+#[test]
+fn test_fffe_issue_6811() {
+    let mut options = CollatorOptions::default();
+    options.strength = Some(Strength::Identical);
+    let collator = Collator::try_new(Default::default(), options).unwrap();
+    // All of these control codes are completely-ignorable, so that
+    // their low code points are compared with the merge separator.
+    // The merge separator must compare less than any other character.
+    assert_eq!(
+        collator.compare_utf16(&[0xFFFE, 0x0001], &[0x0001, 0xFFFE]),
+        core::cmp::Ordering::Less
+    );
+    // The merge separator must even compare less than U+0000.
+    assert_eq!(
+        collator.compare_utf16(&[0xFFFE, 0x0000], &[0x0000, 0xFFFE]),
+        core::cmp::Ordering::Less
+    );
+}
+
+#[cfg(feature = "latin1")]
+#[test]
+fn test_latin1_root() {
+    let mut options = CollatorOptions::default();
+    options.strength = Some(Strength::Primary);
+    let collator = Collator::try_new(Default::default(), options).unwrap();
+    assert_eq!(
+        collator.compare_latin1(b"ax", b"a\xE4"),
+        core::cmp::Ordering::Greater
+    );
+}
+
+#[cfg(feature = "latin1")]
+#[test]
+fn test_latin1_utf16_root() {
+    let mut options = CollatorOptions::default();
+    options.strength = Some(Strength::Primary);
+    let collator = Collator::try_new(Default::default(), options).unwrap();
+    assert_eq!(
+        collator.compare_latin1_utf16(b"ax", &[0x0061, 0x00E4]),
+        core::cmp::Ordering::Greater
+    );
+}
+
+#[cfg(feature = "latin1")]
+#[test]
+fn test_latin1_fi() {
+    let mut options = CollatorOptions::default();
+    options.strength = Some(Strength::Primary);
+    let collator = Collator::try_new(locale!("fi").into(), options).unwrap();
+    assert_eq!(
+        collator.compare_latin1(b"ax", b"a\xE4"),
+        core::cmp::Ordering::Less
+    );
+}
+
+#[cfg(feature = "latin1")]
+#[test]
+fn test_latin1_utf16_fi() {
+    let mut options = CollatorOptions::default();
+    options.strength = Some(Strength::Primary);
+    let collator = Collator::try_new(locale!("fi").into(), options).unwrap();
+    assert_eq!(
+        collator.compare_latin1_utf16(b"ax", &[0x0061, 0x00E4]),
+        core::cmp::Ordering::Less
+    );
+}
+
 // TODO: Test languages that map to the root.
 // The languages that map to root without script reordering are:
 // ca (at least for now)
